<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads.
 *
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2022 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 *  If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Core\Html\Callbacks;

use _JchOptimizeVendor\Joomla\Registry\Registry;
use JchOptimize\Core\FeatureHelpers\LazyLoadExtended;
use JchOptimize\Core\FeatureHelpers\Webp;
use JchOptimize\Core\Helper;
use JchOptimize\Core\Html\ElementObject;
use JchOptimize\Core\Html\Parser;
use JchOptimize\Core\Http2Preload;

\defined('_JCH_EXEC') or exit('Restricted access');
class LazyLoad extends \JchOptimize\Core\Html\Callbacks\AbstractCallback
{
    /**
     * @var bool Used to indicate when the child of a parent element is excluded so the whole element
     *           can be excluded
     */
    public $isExcluded = \false;

    /**
     * @var Http2Preload
     */
    public $http2Preload;

    /**
     * @var array
     */
    protected $excludes;

    /**
     * @var array
     */
    protected $args;

    public function __construct(Registry $params, Http2Preload $http2Preload)
    {
        parent::__construct($params);
        $this->http2Preload = $http2Preload;
        $this->getLazyLoadExcludes();
    }

    public function processMatches($matches)
    {
        if (empty($matches[0])) {
            return $matches[0];
        }
        if (\JCH_PRO && $this->params->get('pro_next_gen_images', '1') && 'picture' != $matches[1]) {
            /** @see Webp::convert() */
            $matches = $this->getContainer()->get(Webp::class)->convert($this->regex, $matches);
        }
        $fullMatch = !empty($matches[0]) ? $matches[0] : \false;
        $elementName = !empty($matches[1]) ? $matches[1] : \false;
        $sClassAttribute = !empty($matches[2]) ? $matches[2] : \false;
        $sClassDelimiter = !empty($matches[3]) ? $matches[3] : '"';
        $sClassValue = !empty($matches[4]) ? $matches[4] : \false;
        $sSrcAttribute = $innerContent = $sStyleAttribute = !empty($matches[5]) ? $matches[5] : \false;
        $sSrcDelimiter = $sStyleDelimiter = !empty($matches[6]) ? $matches[6] : '"';
        $sSrcValue = $sBgDeclaration = !empty($matches[7]) ? $matches[7] : \false;
        $sSrcsetAttribute = $sPosterAttribute = $sCssUrl = !empty($matches[8]) ? $matches[8] : \false;
        $sSrcsetDelimiter = $sPosterDelimiter = $sCssUrlValue = !empty($matches[9]) ? $matches[9] : \false;
        $sSrcsetValue = $sPosterValue = !empty($matches[10]) ? $matches[10] : \false;
        $sAutoLoadAttribute = $sPreloadAttribute = $sWidthAttribute = !empty($matches[11]) ? $matches[11] : \false;
        $sWidthDelimiter = $sPreloadDelimiter = !empty($matches[12]) ? $matches[12] : \false;
        $sWidthValue = $sPreloadValue = !empty($matches[13]) ? $matches[13] : 1;
        $sHeightAttribute = $sAutoPlayAttribute = !empty($matches[14]) ? $matches[14] : \false;
        $sHeightDelimiter = $sAutoPlayDelimiter = !empty($matches[15]) ? $matches[15] : \false;
        $sHeightValue = $sAutoPlayValue = !empty($matches[16]) ? $matches[16] : 1;
        $isLazyLoaded = \false;
        // Return match if it isn't an HTML element
        if (\false === $elementName) {
            return $fullMatch;
        }

        switch ($elementName) {
            case 'img':
            case 'input':
            case 'picture':
            case 'iframe':
            case 'source':
                $sImgType = 'embed';

                break;

            case 'video':
            case 'audio':
                $sImgType = 'audiovideo';

                break;

            default:
                $sImgType = 'background';

                break;
        }
        if ($this->args['lazyload']) {
            if ('img' == $elementName || 'input' == $elementName) {
                $this->http2Preload->add($sSrcValue, 'image', \true);
            }
            // Start modifying the element to return
            $return = $fullMatch;
            // Exclude based on class
            if (\false !== $sClassValue) {
                if ($elementName && Helper::findExcludes($this->excludes['class'], $sClassValue)) {
                    // If element child of a parent element set excluded flag
                    if ('' != $this->args['parent']) {
                        $this->isExcluded = \true;
                    }

                    return $fullMatch;
                }
            }
            if ('picture' != $elementName) {
                // If a src attribute is found
                if (\false !== $sSrcAttribute) {
                    $sImgName = 'background' == $sImgType ? $sCssUrlValue : $sSrcValue;
                    // Abort if this file is excluded
                    if (Helper::findExcludes($this->excludes['url'], $sImgName)) {
                        // If element child of a parent element set excluded flag
                        if ('' != $this->args['parent']) {
                            $this->isExcluded = \true;
                        }

                        return $fullMatch;
                    }
                    // If no srcset attribute was found, modify the src attribute and add a data-src attribute
                    if (\false === $sSrcsetAttribute && 'embed' == $sImgType) {
                        $sSvg = '<svg xmlns="http://www.w3.org/2000/svg" width="'.$sWidthValue.'" height="'.$sHeightValue.'"></svg>';
                        $sNewSrcValue = 'iframe' == $elementName ? 'about:blank' : 'data:image/svg+xml;base64,'.\base64_encode($sSvg);
                        $sNewSrcAttribute = 'src='.$sSrcDelimiter.$sNewSrcValue.$sSrcDelimiter.' data-'.$sSrcAttribute;
                        $return = \str_replace($sSrcAttribute, $sNewSrcAttribute, $return);
                        $isLazyLoaded = \true;
                    }
                }
                // If poster attribute was found we can also exclude using poster value
                if (\false !== $sPosterAttribute) {
                    if (Helper::findExcludes($this->excludes['url'], $sPosterValue)) {
                        return $fullMatch;
                    }
                }
                // Modern browsers will lazy-load without loading the src attribute
                if (\false !== $sSrcsetAttribute && 'embed' == $sImgType) {
                    $sSvgSrcset = '<svg xmlns="http://www.w3.org/2000/svg" width="'.$sWidthValue.'" height="'.$sHeightValue.'"></svg>';
                    $sSrcsetDelimiter = $sSrcsetDelimiter ?: '"';
                    $sNewSrcsetAttribute = 'srcset='.$sSrcsetDelimiter.'data:image/svg+xml;base64,'.\base64_encode($sSvgSrcset).$sSrcsetDelimiter.' data-'.$sSrcsetAttribute;
                    $return = \str_replace($sSrcsetAttribute, $sNewSrcsetAttribute, $return);
                    $isLazyLoaded = \true;
                }
                if (\JCH_PRO && 'audiovideo' == $sImgType) {
                    /** @see LazyLoadExtended::lazyLoadAudioVideo() */
                    $return = $this->getContainer()->get(LazyLoadExtended::class)->lazyLoadAudioVideo($matches, $return);
                    $isLazyLoaded = \true;
                }
            }
            // Process and add content of element if not self-closing
            if ('picture' == $elementName && \false !== $innerContent) {
                $args = ['lazyload' => \true, 'deferred' => \true, 'parent' => 'picture'];
                $sInnerContentLazyLoaded = $this->lazyLoadInnerContent($innerContent, $args);
                // If any child element were lazyloaded this function will return false
                if (\false === $sInnerContentLazyLoaded) {
                    return $fullMatch;
                }

                return \str_replace($innerContent, $sInnerContentLazyLoaded, $fullMatch);
            }
            if (\JCH_PRO && 'background' == $sImgType && $this->params->get('pro_lazyload_bgimages', '0')) {
                /** @see LazyLoadExtended::lazyLoadBgImages() */
                $return = $this->getContainer()->get(LazyLoadExtended::class)->lazyLoadBgImages($matches, $return);
                $isLazyLoaded = \true;
            }
            if ($isLazyLoaded) {
                // If class attribute not on the appropriate element add it
                if ('source' != $elementName && \false === $sClassAttribute) {
                    $return = \str_replace('<'.$elementName, '<'.$elementName.' class="jch-lazyload"', $return);
                }
                // If class already on element add the lazy-load class
                if ('source' != $elementName && \false !== $sClassAttribute) {
                    $sNewClassAttribute = 'class='.$sClassDelimiter.$sClassValue.' jch-lazyload'.$sClassDelimiter;
                    $return = \str_replace($sClassAttribute, $sNewClassAttribute, $return);
                }
            }
            if ('picture' != $this->args['parent'] && $isLazyLoaded) {
                // Wrap and add img elements in noscript
                if ('img' == $elementName || 'iframe' == $elementName) {
                    $return .= '<noscript>'.$fullMatch.'</noscript>';
                }
            }

            return $return;
        }
        if (\false !== $sSrcAttribute && ('img' == $elementName || 'input' == $elementName)) {
            $this->http2Preload->add($sSrcValue, 'image', $this->args['deferred']);
        }
        if ('background' == $sImgType && \false !== $sStyleAttribute) {
            $this->http2Preload->add($sCssUrlValue, 'image', $this->args['deferred']);
        }
        // If lazy-load enabled, remove loading="lazy" attributes from above the fold
        if ($this->params->get('lazyload_enable', '0') && !$this->args['deferred'] && 'img' == $elementName) {
            // Remove any lazy loading
            $fullMatch = \preg_replace('#loading\\s*+=\\s*+["\']?lazy["\']?#i', '', $fullMatch);
        }
        // We may need to convert images to WEBP in picture elements
        if ('picture' == $elementName && \false !== $innerContent) {
            $args = ['lazyload' => \false, 'deferred' => $this->args['deferred'], 'parent' => 'picture'];
            $innerContentWebp = $this->lazyLoadInnerContent($innerContent, $args);
            if (\false !== $innerContentWebp) {
                $fullMatch = \str_replace($innerContent, $innerContentWebp, $fullMatch);
            }
        }

        return $fullMatch;
    }

    public function setLazyLoadArgs($args)
    {
        $this->args = $args;
    }

    protected function getLazyLoadExcludes()
    {
        $aExcludesFiles = Helper::getArray($this->params->get('excludeLazyLoad', []));
        $aExcludesFolders = Helper::getArray($this->params->get('pro_excludeLazyLoadFolders', []));
        $aExcludesUrl = \array_merge(['data:image'], $aExcludesFiles, $aExcludesFolders);
        $aExcludeClass = Helper::getArray($this->params->get('pro_excludeLazyLoadClass', []));
        $this->excludes = ['url' => $aExcludesUrl, 'class' => $aExcludeClass];
    }

    protected function lazyLoadInnerContent($innerContent, array $args)
    {
        $oParser = new Parser();
        $oImgElement = new ElementObject();
        $oImgElement->bSelfClosing = \true;
        $oImgElement->setNamesArray(['img', 'source']);
        // language=RegExp
        $oImgElement->addNegAttrCriteriaRegex('(?:data-(?:src|original))');
        $oImgElement->setCaptureAttributesArray(['class', 'src', 'srcset', '(?:data-)?width', '(?:data-)?height']);
        $oParser->addElementObject($oImgElement);

        /** @var LazyLoad $lazyLoadCallback */
        $lazyLoadCallback = $this->getContainer()->get(\JchOptimize\Core\Html\Callbacks\LazyLoad::class);
        $lazyLoadCallback->setLazyLoadArgs($args);
        $result = $oParser->processMatchesWithCallback($innerContent, $lazyLoadCallback);
        // if any child element were excluded return false
        if ($lazyLoadCallback->isExcluded) {
            return \false;
        }

        return $result;
    }
}
