<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads.
 *
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2022 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 *  If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Core\Service;

use _JchOptimizeVendor\Interop\Container\ContainerInterface as InteropContainerInterface;
use _JchOptimizeVendor\Joomla\DI\Container;
use _JchOptimizeVendor\Joomla\DI\ServiceProviderInterface;
use _JchOptimizeVendor\Laminas\EventManager\LazyListener;
use _JchOptimizeVendor\Laminas\EventManager\SharedEventManager;
use _JchOptimizeVendor\Laminas\ServiceManager\PsrContainerDecorator;
use JchOptimize\Core\Cdn;
use JchOptimize\Core\FeatureHelpers\CdnDomains;
use JchOptimize\Core\FeatureHelpers\DynamicJs;
use JchOptimize\Core\FeatureHelpers\DynamicSelectors;
use JchOptimize\Core\FeatureHelpers\Fonts;
use JchOptimize\Core\FeatureHelpers\Http2Excludes;
use JchOptimize\Core\FeatureHelpers\LazyLoadExtended;
use JchOptimize\Core\FeatureHelpers\ReduceDom;
use JchOptimize\Core\FeatureHelpers\Webp;
use JchOptimize\Core\Html\CacheManager;
use JchOptimize\Core\Html\LinkBuilder;
use JchOptimize\Core\Http2Preload;

\defined('_JCH_EXEC') or exit('Restricted access');
class FeatureHelpersProvider implements ServiceProviderInterface
{
    public function register(Container $container)
    {
        $container->share(CdnDomains::class, function (Container $container): CdnDomains {
            return (new CdnDomains($container->get('params'), $container->get(Cdn::class), $container->get(LinkBuilder::class)))->setContainer($container);
        }, \true);
        $container->share(DynamicJs::class, function (Container $container): DynamicJs {
            return (new DynamicJs($container->get('params'), $container->get(CacheManager::class), $container->get(LinkBuilder::class)))->setContainer($container);
        }, \true);
        $container->share(DynamicSelectors::class, function (Container $container): DynamicSelectors {
            return (new DynamicSelectors($container->get('params')))->setContainer($container);
        }, \true);
        $container->share(Fonts::class, function (Container $container): Fonts {
            return (new Fonts($container->get('params'), $container->get(LinkBuilder::class)))->setContainer($container);
        }, \true);
        $container->share(Http2Excludes::class, function (Container $container): Http2Excludes {
            return (new Http2Excludes($container->get('params'), $container->get(Http2Preload::class)))->setContainer($container);
        }, \true);
        $container->share(LazyLoadExtended::class, function (Container $container): LazyLoadExtended {
            return (new LazyLoadExtended($container->get('params'), $container->get(LinkBuilder::class)))->setContainer($container);
        }, \true);
        $container->share(ReduceDom::class, function (Container $container): ReduceDom {
            return (new ReduceDom($container->get('params')))->setContainer($container);
        }, \true);
        $container->share(Webp::class, function (Container $container): Webp {
            return (new Webp($container->get('params')))->setContainer($container);
        }, \true);
        // Set up events management
        /** @var SharedEventManager $sharedEvents */
        $sharedEvents = $container->get(SharedEventManager::class);

        /** @var InteropContainerInterface $interopContainer */
        $interopContainer = new PsrContainerDecorator($container);
        $sharedEvents->attach(LinkBuilder::class, 'preProcessHtml', new LazyListener([
            // @see Fonts::checkPreconnects()
            'listener' => Fonts::class,
            'method' => 'checkPreconnects',
        ], $interopContainer));
        $sharedEvents->attach(LinkBuilder::class, 'postProcessHtml', new LazyListener([
            // @see Fonts::appendOptimizedFontsToHead()
            'listener' => Fonts::class,
            'method' => 'appendOptimizedFontsToHead',
        ], $interopContainer));
        $sharedEvents->attach(LinkBuilder::class, 'postProcessHtml', new LazyListener([
            // @see Fonts::addPreConnectsFontsToHead()
            'listener' => Fonts::class,
            'method' => 'addPreConnectsFontsToHead',
        ], $interopContainer), -100);
        $sharedEvents->attach(LinkBuilder::class, 'preProcessHtml', new LazyListener([
            // @see LazyLoadExtended::addCssLazyLoadAssetsToHtml()
            'listener' => LazyLoadExtended::class,
            'method' => 'addCssLazyLoadAssetsToHtml',
        ], $interopContainer));
        $sharedEvents->attach(LinkBuilder::class, 'postProcessHtml', new LazyListener([
            // @see LazyLoadExtended::lazyLoadCssBackgroundImages()
            'listener' => LazyLoadExtended::class,
            'method' => 'lazyLoadCssBackgroundImages',
        ], $interopContainer));
    }
}
